﻿// Disclaimer:
// This software is provided AS IS for educational purposes only,
// with no warranties, and confers no rights. Error checking is 
// elided for clarity. Carry out sufficient tests prior to using
// it in a productive environment.
// CASIO Europe GmbH accepts no responsibility for loss or damage
// arising from its use, including damage from virus.
//
// Please, note: This configuration information may be changed
// without any further notice.
// Casio Europe GmbH does not guarantee future compatibility. By 
// using this software you agree to these terms and conditions.
//
// Version 1.01 - April 18, 2012
// Supported Devices: DT-X30, IT-800, IT-9000
//
// Remarks: You need latest patches and runtime libraries installed
// onto your mobile device.
//
//////////////////////////////////////////////////////////////////////

using System;

using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.WindowsCE.Forms;
using System.Globalization;
using System.Reflection;
using Calib;

namespace CameraSample
{
    #region Enums

    // Supported devices.
    public enum ModelName
    {
        None, 
        DTX30CE,
        DTX30WM,
        IT800WM,
        IT9000CE,
        IT9000WM
    }
    
    // Error that can occur during initialize.
    public enum InitError
    {
        // Success.
        None,
        // Model is not supported.
        Model,
        // Hot key is not available, e.g. it is used by other application.
        HotKey,
        // Camera cannot be started.
        CamOpen
    }

    #endregion

    // This application shows how to take a picture by pressing the center trigger key.
    // Besides, it demonstrates how to handle screen rotation.
    public partial class MainForm : Form
    {
        #region Fields

        // Initial keyboard driver state that must be restored.
        private bool initUserDefineKeyState;
        // Status of preview.
        private bool preview;
        // Dimensions of preview.
        private int previewSize;
        // Rotation of preview.
        private int previewRotation = CameraLibNet.Def.CAM_ROTATE_NONE;
        // Dimensions of panel containing the preview.
        private Size previewContainerSize;
        // Actual platform determined at runtime.
        private ModelName currentModel;
        // Current screen orientation.
        private ScreenOrientation currentScreenOrientation = ScreenOrientation.Angle0;
        // Initial screen orientation that must be restored.
        private ScreenOrientation initScreenOrientation;
        // Message Windows that receives windows messages.
        private MessageProcessing messageProcessing;

        #endregion

        #region Settings

        // QVGA Screen
        private readonly Size vga1per9 = new Size(160, 213);
        // VGA Screen
        private readonly Size vga4per9 = new Size(320, 432);

        // Dimensions of the picture to be taken.
        private const int DefaultResolution = CameraLibNet.Def.CAM_VGA;
        // File format of the picture to be taken.
        private const int DefaultFormat = CameraLibNet.Def.CAM_JPEG;
        // Default path and file name of the picture to be taken.
        private const string DefaultPath = "\\My Documents\\CameraSample.jpg";
        // Trigger key that activates the auto focus.
        private const int TriggerKeyLeft = SystemLibNet.Def.KEYID_LEFTTRIGGER;
        // Internal ID of the left trigger key.
        private const int IdHotKeyLeft = 0xD000;
        // Trigger key that activates the shutter.
        private const int TriggerKeyRight = SystemLibNet.Def.KEYID_RIGHTTRIGGER;
        // Internal ID of the right trigger key.
        private const int IdHotKeyRight = IdHotKeyLeft + 1;

        #endregion

        #region Init/Deinit

        // Initialize.
        public MainForm()
        {
            InitializeComponent();

            // We need full speed for processing graphics.
            SystemLibNet.Api.SysSetCPUMode(SystemLibNet.Def.CPUMODE_HIGH);

            // Init settings.
            if (SystemLibNet.Api.SysGetUserDefineKeyState() == 1)
            {
                this.initUserDefineKeyState = true;
            }
            else
            {
                this.initUserDefineKeyState = false;
            }
            SystemLibNet.Api.SysSetUserDefineKeyState(false);
            UpdatePreviewRotation();
            this.initScreenOrientation = SystemSettings.ScreenOrientation;
            this.messageProcessing = new MessageProcessing(this);

            // Make regional settings.
            Resources.Culture = CultureInfo.CurrentCulture;
            Text = Resources.Title;
            this.menuHelp.Text = Resources.MenuHelp;
            this.menuRotate.Text = Resources.MenuRotate;

            // Make further settings depending on actual model at runtime. 
            int model = 0;
            int version = 0;
            int platform = 0;

            if (SystemLibNet.Api.SysGetModelName(ref model, ref version, ref platform) == -1)
            {
                switch (model)
                {
                    case 10:
                        this.currentModel = ModelName.DTX30CE;
                        this.previewSize = CameraLibNet.Def.CAM_1PER9VGA;
                        this.previewContainerSize = this.vga1per9;
                        WindowState = FormWindowState.Maximized;
                        break;
                    case 11:
                        this.currentModel = ModelName.DTX30WM;
                        this.previewSize = CameraLibNet.Def.CAM_1PER9VGA;
                        this.previewContainerSize = this.vga1per9;
                        WindowState = FormWindowState.Normal;
                        break;
                    case 13:
                        this.currentModel = ModelName.IT800WM;
                        this.previewSize = CameraLibNet.Def.CAM_4PER9VGA;
                        this.previewContainerSize = this.vga4per9;
                        WindowState = FormWindowState.Normal;
                        break;
                    case 19:
                        this.currentModel = ModelName.IT9000CE;
                        this.previewSize = CameraLibNet.Def.CAM_1PER9VGA;
                        this.previewContainerSize = this.vga1per9;
                        WindowState = FormWindowState.Maximized;
                        break;
                    case 20:
                        this.currentModel = ModelName.IT9000WM;
                        this.previewSize = CameraLibNet.Def.CAM_4PER9VGA;
                        this.previewContainerSize = this.vga4per9;
                        WindowState = FormWindowState.Normal;
                        break;
                    default:
                        this.currentModel = ModelName.None;
                        break;
                }
            }

            // Prepare error message.
            Message msg = new Message();
            msg.HWnd = this.messageProcessing.Hwnd;
            msg.Msg = MessageProcessing.InitMessage;

            // Register hot key and open camera.
            if (this.currentModel != ModelName.None)
            {
                int defaultKeyLeft = GetDefaultKey(TriggerKeyLeft);
                int defaultKeyRight = GetDefaultKey(TriggerKeyRight);

                if (defaultKeyLeft != 0 && defaultKeyRight != 0 &&
                    RegisterHotKey(this.messageProcessing.Hwnd, IdHotKeyLeft, defaultKeyLeft) == true &&
                    RegisterHotKey(this.messageProcessing.Hwnd, IdHotKeyRight, defaultKeyRight) == true)
                {
                    if (CameraLibNet.Api.CAMOpen() == CameraLibNet.Def.CAM_SUCCESS)
                    {
                        CameraLibNet.Api.CAMSetPreviewMode(
                            CameraLibNet.Def.CAM_PREVIEW_FOCUSFRAME, 0);

                        msg.WParam = new IntPtr((int)InitError.None);
                    }
                    else
                    {
                        msg.WParam = new IntPtr((int)InitError.CamOpen);
                    }
                }
                else
                {
                    msg.WParam = new IntPtr((int)InitError.HotKey);
                }
            }
            else
            {
                msg.WParam = new IntPtr((int)InitError.Model);
            }

            MessageWindow.PostMessage(ref msg);
        }

        // Clean-up.
        private void MainForm_Closing(object sender, CancelEventArgs e)
        {
            UnregisterHotKey(this.messageProcessing.Hwnd, IdHotKeyLeft);
            UnregisterHotKey(this.messageProcessing.Hwnd, IdHotKeyRight);
            StopPreview();
            CameraLibNet.Api.CAMClose();
            if (this.initUserDefineKeyState == true)
            {
                SystemLibNet.Api.SysSetUserDefineKeyState(true);
            }
            SystemSettings.ScreenOrientation = this.initScreenOrientation;
        }

        #endregion

        #region Message Handler

        // Handle message from initalize.
        public void OnInit(InitError error)
        {
            if (error == InitError.None)
            {
                StartPreview();
            }
            else
            {
                string msg;

                switch (error)
                {
                    case InitError.CamOpen:
                        msg = Resources.ErrorCamOpen;
                        break;
                    case InitError.HotKey:
                        msg = Resources.ErrorHotKey;
                        break;
                    case InitError.Model:
                        msg = Resources.ErrorModel;
                        break;
                    default:
                        msg = Resources.ErrorUnknown;
                        break;
                }

                MessageBox.Show(msg, Resources.CaptionError);
            }
        }

        // Handle screen rotation.
        public void OnSettingChangeReset()
        {
            UpdatePreviewRotation();
            StopPreview();
            // Refresh full-screen mode on Windows Embedded CE only.
            switch (this.currentModel)
            {
                case ModelName.DTX30CE:
                case ModelName.IT9000CE:
                    WindowState = FormWindowState.Normal;
                    WindowState = FormWindowState.Maximized;
                    break;
                default:
                    break;
            }
            StartPreview();
        }

        // Handle message from trigger key (hot key).
        public void OnTriggerKey(int hotKey)
        {
            if (hotKey == IdHotKeyLeft)
            {
                int reserved = 0;
                int result = CameraLibNet.Api.CAMAutoFocus(
                    CameraLibNet.Def.CAM_AF_NORMAL,
                    ref reserved);

                if (result != CameraLibNet.Def.CAM_SUCCESS)
                {
                    SystemLibNet.Api.SysPlayBuzzer(
                        SystemLibNet.Def.B_ALARM,
                        SystemLibNet.Def.BUZ_DEFAULT,
                        SystemLibNet.Def.BUZ_DEFAULT);
                }
            }
            else if (hotKey == IdHotKeyRight)
            {
                Cursor.Current = Cursors.WaitCursor;

                try
                {
                    CameraLibNet.Api.CAMSingleCapture(
                        DefaultPath,
                        DefaultFormat,
                        DefaultResolution);
                }
                finally
                {
                    Cursor.Current = Cursors.Default;
                }

            }

            // Remove other messages.
            NativeMethods.MSG peekMsg = new NativeMethods.MSG();
            do
            {
                if (NativeMethods.PeekMessage(
                    peekMsg,
                    IntPtr.Zero,
                    NativeMethods.WM_HOTKEY,
                    NativeMethods.WM_HOTKEY,
                    NativeMethods.PM_REMOVE) == 0)
                {
                    break;
                }

            } while (true);
        }

        #endregion

        #region Menu Handler

        // Rotate the screen by 90 degrees.
        private void menuRotate_Click(object sender, EventArgs e)
        {
            StopPreview();

            if (SystemSettings.ScreenOrientation == ScreenOrientation.Angle0)
            {
                SystemSettings.ScreenOrientation = ScreenOrientation.Angle90;
            }
            else
            {
                SystemSettings.ScreenOrientation = ScreenOrientation.Angle0;
            }
        }

        // Display some instructions to the user.
        private void menuInfo_Click(object sender, EventArgs e)
        {
            MessageBox.Show(Resources.MessageInfo, Resources.MenuHelp);
        }

        // Display version info.
        private void menuAbout_Click(object sender, EventArgs e)
        {
            string[] tokens = Assembly.GetExecutingAssembly().ToString().Split(new char[] { ',' });
            if (tokens.Length >= 2)
            {
                MessageBox.Show(tokens[0] + tokens[1], Resources.MenuHelp);
            }
        }

        #endregion

        #region Preview

        // Open preview.
        private void StartPreview()
        {
            // Center preview.
            int xPos, yPos;

            if (this.currentScreenOrientation == ScreenOrientation.Angle0 ||
                this.currentScreenOrientation == ScreenOrientation.Angle180)
            {
                xPos = (this.pnlPreview.Size.Width - this.previewContainerSize.Width) / 2;
                yPos = (this.pnlPreview.Size.Height - this.previewContainerSize.Height) / 2 + 1;
            }
            else
            {
                xPos = (this.pnlPreview.Size.Width - this.previewContainerSize.Height) / 2;
                yPos = (this.pnlPreview.Size.Height - this.previewContainerSize.Width) / 2 + 1;
            }

            int res = this.previewSize | this.previewRotation;

            if (CameraLibNet.Api.CAMStartPreview(
                this.pnlPreview.Handle,
                xPos,
                yPos,
                res) == Calib.CameraLibNet.Def.CAM_SUCCESS)
            {
                this.preview = true;
            }
        }

        // Close preview.
        private void StopPreview()
        {
            if (this.preview == true &&
                CameraLibNet.Api.CAMStopPreview() == Calib.CameraLibNet.Def.CAM_SUCCESS)
            {
                this.preview = false;
            }
        }

        #endregion

        #region Helper Methods

        // Update settings.
        private void pnlPreview_Resize(object sender, EventArgs e)
        {
            if (this.preview == true)
            {
                // Windows Mobile provides full-screen mode only. Hence, we have to handle 
                // resizing for Windows Embedded CE only.
                switch (this.currentModel)
                {
                    case ModelName.DTX30CE:
                    case ModelName.IT9000CE:
                        StopPreview();
                        StartPreview();
                        break;
                    default:
                        break;
                }
            }
        }

        // Determine screen rotation.
        private void UpdatePreviewRotation()
        {
            ScreenOrientation tempOrientation = SystemSettings.ScreenOrientation;

            if (this.currentScreenOrientation != tempOrientation)
            {
                this.currentScreenOrientation = tempOrientation;

                switch (this.currentScreenOrientation)
                {
                    case ScreenOrientation.Angle90:
                        this.previewRotation = CameraLibNet.Def.CAM_ROTATE_270;
                        break;
                    case ScreenOrientation.Angle180:
                        this.previewRotation = CameraLibNet.Def.CAM_ROTATE_180;
                        break;
                    case ScreenOrientation.Angle270:
                        this.previewRotation = CameraLibNet.Def.CAM_ROTATE_90;
                        break;
                    default:
                        this.previewRotation = CameraLibNet.Def.CAM_ROTATE_NONE;
                        break;
                }
            }
        }

        #endregion

        #region Trigger Key

        // Register hot key.
        private bool RegisterHotKey(IntPtr hwnd, int id, int vk)
        {
            bool result = false;

            if (NativeMethods.RegisterHotKey(
                hwnd,
                id,
                0,
                (uint)vk) != 0)
            {
                result = true;
            }

            return result;
        }
        
        // Unregister hot key.
        private bool UnregisterHotKey(IntPtr hwnd, int id)
        {
            bool result = false;

            if (NativeMethods.UnregisterHotKey(
                hwnd,
                id) != 0)
            {
                result = true;
            }

            return result;
        }

        // Determine key code at run-time. It depends on actual target device.
        public int GetDefaultKey(int keyId)
        {
            int result = 0;
            int[] buffer = new int[16];

            if (SystemLibNet.Api.SysGetDefaultKey(keyId, buffer) == -1)
            {
                result = buffer[0] & 0xFF;
            }

            return result;
        }

        #endregion
    }
}